/**
 * Copyright (c) 2013 The Chromium Authors. All Rights Reserved.
 * Copyright (c) 2015 Nexedi SA and Contributors. All Rights Reserved.
 *                       Romain Courteaud <romain@nexedi.com>
 * Use of this source code is governed by a BSD-style license that can be
 * found in the LICENSE file.
 **/

function $(id) {
  return document.getElementById(id);
}

function updateStatus(socket_dict) {
  var key,
    count = 0,
    text = "";

  for (key in socket_dict) {
    if (socket_dict.hasOwnProperty(key)) {
      text += socket_dict[key].peer + "\n";
      count += 1;
    }
  }

  $('peer_list').value = text;
  $('client_count').textContent = count;
}

function showAccessUrl(port) {
  chrome.system.network.getNetworkInterfaces(function(result_list) {
    var i,
      text = "";
    if (result_list) {
      for (i = 0; i < result_list.length; i++) {
        if (result_list[i].prefixLength == 24) {
          text += 'ws://' + result_list[i].address + ':' + port + '/';
        }
      }
      document.getElementById('server_url_list').value = text;
    }
  });
}

var port = 9999;
if (http.Server && http.WebSocketServer) {
  // Listen for HTTP connections.
  var server = new http.Server();
  var wsServer = new http.WebSocketServer(server);
  server.listen(port);

  showAccessUrl(port);

  server.addEventListener('request', function(req) {
    var url = req.headers.url;
    if (url == '/')
      url = '/index.html';
    // Serve the pages of this chrome application.
    req.serveUrl(url);
    return true;
  });

  // A list of connected websockets.
  var socket_dict = {};

  wsServer.addEventListener('request', function(req) {
    var socket = req.accept();
    socket_dict[socket.socketId_] = {
      socket: socket
    };
    chrome.socket.getInfo(socket.socketId_, function (result) {
      if ((result !== undefined) && (result.connected)) {
        socket_dict[socket.socketId_].peer = result.peerAddress + ':' + result.peerPort;
        updateStatus(socket_dict);
      }
    });

    // When a message is received on one socket, rebroadcast it on all
    // connected sockets.
    socket.addEventListener('message', function(e) {
      var key;
      for (key in socket_dict) {
        if (socket_dict.hasOwnProperty(key)) {
          socket_dict[key].socket.send(e.data);
        }
      }
    });

    // When a socket is closed, remove it from the list of connected sockets.
    socket.addEventListener('close', function() {
      delete socket_dict[socket.socketId_];
      updateStatus(socket_dict);
    });
    return true;
  });
}
